from collections import deque, defaultdict

def bfs(start, max_distance, graph, n):
    visited = [False] * (n + 1)
    queue = deque([(start, 0)])  # (current_node, current_distance)
    visited[start] = True
    protected_stores = set()

    while queue:
        current, distance = queue.popleft()
        protected_stores.add(current)

        if distance < max_distance:
            for neighbor in graph[current]:
                if not visited[neighbor]:
                    visited[neighbor] = True
                    queue.append((neighbor, distance + 1))

    return protected_stores

def main():
    n, m, k = map(int, input().split())
    graph = defaultdict(list)

    # Read roads
    for _ in range(m):
        a, b = map(int, input().split())
        graph[a].append(b)
        graph[b].append(a)

    protected_stores = set()

    # Read guards
    for _ in range(k):
        p, h = map(int, input().split())
        protected_stores.update(bfs(p, h, graph, n))

    # Output results
    protected_stores = sorted(protected_stores)
    print(len(protected_stores))
    print(" ".join(map(str, protected_stores)))

if __name__ == "__main__":
    main()
